from . import ContainerImageList, ContainerImage
import logging
import os

from docker import DockerClient

logger = logging.getLogger("otimages." + os.path.basename(__file__))


class xECMMonImages(ContainerImageList):
    def __init__(
        self,
        version: str,
        docker_client: DockerClient,
        repository: str = "artifactory.otxlab.net",
        path: str = "/ot2-paas-dev/ecmcontainerization/",
        latest: bool = False,
    ):
        self.version = version
        self.repository = repository
        self.latest = latest
        self.docker_client = docker_client

        stage = "latest" if self.latest else "released"

        self.path = path + stage

        self.images = self.__get_images()

    def __get_images(self):
        all_images = []
        all_images.append(
            ContainerImage(
                repository=self.repository,
                path=self.path,
                name="xecm-mon",
                version="latest" if self.latest else self.version,
                version_overwrite="stable",
                docker_client=self.docker_client,
            )
        )
        return all_images



from . import ContainerImageList, ContainerImage
import logging
import os
from docker import DockerClient

from .helm import run_helm_command

logger = logging.getLogger("otimages." + os.path.basename(__file__))


class AppworksGatewayImages(ContainerImageList):
    def __init__(
        self,
        docker_client: DockerClient,
        version: str,
        repository: str = "artifactory.otxlab.net",
        path: str = "/ot2-paas/otag/",
        helm_base_url: str = "https://artifactory.otxlab.net/artifactory/ot2-helm/otag/helm/release-helm",
        helm_dir: str = "",
        latest: bool = False,
    ):
        self.version = version
        self.repository = repository
        self.docker_client = docker_client
        self.helm_base_url = helm_base_url
        self.path = path
        self.latest = latest

        if helm_dir == "":
            self.helm_dir = (
                self.helm_base_url + "/appworks-gateway-" + self.version + ".tar.gz"
            )
        else:
            self.helm_dir = helm_dir

        self.images = self.__get_images()

    def __get_images(self):

        logger.info(
            "Running helm template for %s in %s",
            self.version,
            self.helm_dir,
        )

        command = f"helm template {self.helm_dir} --set database.appworksdb.password=Opentext1!,awg.admin.newadminpassword=Opentext1!,otds.admin.password=Opentext1!,global.imageSource=registry.opentext.com| yq '..|.image? | select(.)' | sort -u | grep registry.opentext.com | sed 's/registry.opentext.com\///g'"
        images = run_helm_command(command)
        all_images = []

        for image in images:
            name = image.split(":")[0]
            version = image.split(":")[1]
            version_overwrite = version

            all_images.append(
                ContainerImage(
                    repository=self.repository,
                    path=self.path,
                    name=name,
                    version=version,
                    version_overwrite=version_overwrite,
                    docker_client=self.docker_client,
                )
            )
        if images:
            all_images.append(
                ContainerImage(
                    repository=self.repository,
                    path="/ot2-paas/ecmcontainerization/released/",
                    name="otawg-csmobile-init",
                    version=self.version,
                    version_overwrite=self.version,
                    docker_client=self.docker_client,
                )
            )

        return all_images



from . import ContainerImageList, ContainerImage
import logging
import os
import tempfile
import requests
import tarfile
import typer

from docker import DockerClient

from .helm import run_helm_command

logger = logging.getLogger("otimages." + os.path.basename(__file__))


class xECMImages(ContainerImageList):
    def __init__(
        self,
        docker_client: DockerClient,
        version: str,
        repository: str = "artifactory.otxlab.net",
        path: str = "/ot2-paas-dev/ecmcontainerization/",
        helm_base_url: str = "https://artifactory.otxlab.net/artifactory/ot2-helm-dev/ecm/",
        helm_dir: str = "",
        langpacks: bool = False,
        latest: bool = False,
    ):
        self.version = version
        self.repository = repository
        self.latest = latest
        self.langpacks = langpacks
        self.docker_client = docker_client

        self.stage = "latest" if self.latest else "released"

        if (
            helm_base_url
            == "https://artifactory.otxlab.net/artifactory/ot2-helm-dev/ecm/"
        ):
            self.helm_base_url = helm_base_url + self.stage
        else:
            self.helm_base_url = helm_base_url

        self.path = path + self.stage

        if helm_dir == "":
            self.helm_dir = self.__get_helm_dir()
        else:
            self.helm_dir = helm_dir

        self.images = self.__get_images()

    def __get_helm_dir(self):
        temp_dir = tempfile.mkdtemp()
        helm_tgz = temp_dir + "/otxecm.tgz"

        logger.debug("Creating temporary directory for helm %s ", temp_dir)

        if self.latest:
            url = self.helm_base_url + "/otxecm/otxecm-" + self.version + "-latest.tgz"
        else:
            url = self.helm_base_url + "/otxecm-" + self.version + ".tgz"

        logger.debug("Downloading Helm Chart from %s", url)

        try:
            r = requests.get(url, timeout=10)
            with open(helm_tgz, "wb") as f:
                f.write(r.content)

            # open file
            file = tarfile.open(helm_tgz)
            file.extractall(temp_dir)
            file.close()

        except tarfile.ReadError:
            logger.error("Helm chart could not be downloaded from: %s", url)
            raise typer.Exit(code=1)

        except Exception as err:
            raise err

        return temp_dir + "/otxecm"

    def __get_images(self):
        logger.debug(
            "Updating %s/platforms/azure.yaml with .global.masterPassword",
            self.helm_dir,
        )

        platformfile = f"{self.helm_dir}/platforms/azure.yaml"
        if os.path.isfile(platformfile):
            os.popen(
                f"yq -i '.global.masterPassword = \"Opentext1!\"' {platformfile}"
            ).readlines()
        else:
            logger.error("%s does not exist, skipping update", platformfile)

        logger.info(
            "Running helm template for %s in %s",
            self.version,
            self.helm_dir,
        )
        if os.path.isfile(f"{self.helm_dir}/otxecm-image-tags.yaml"):
            tagsfile = f"-f {self.helm_dir}/otxecm-image-tags.yaml"
        else:
            tagsfile = ""

        if os.path.isfile(f"{self.helm_dir}/platforms/azure.yaml"):
            platformfile = f"-f {self.helm_dir}/platforms/azure.yaml"
        else:
            platformfile = ""

        command = f"helm template {self.helm_dir} {tagsfile} {platformfile} --set global.masterPassword=Opentext1!,global.otpd.enabled=true,otpd.loadLicense=false,global.ingressEnabled=false --set otcs.config.documentStorage.type=database --set otiv.amqp.rabbitmq.password=test --set otds.otdsws.cryptKey=12dghfgdh3 --set otds.otdsws.otdsdb.username=otds --set otac.database.port=5432 --set otac.database.name=ac --set otac.database.hostname=otxecm-db --set otac.database.username=ac --set otds.otdsws.otdsdb.password=Opentext1! --set otpd.technicalUserPassword=Opentext1! --set otcs.serviceAccountName=test --set otcs.serviceType=ClusterIP | yq '..|.image? | select(.)' | sort -u | grep registry.opentext.com | sed 's/registry.opentext.com\///g'"
        images = run_helm_command(command)

        all_images = []

        for image in images:
            name = image.split(":")[0]
            version = image.split(":")[1]
            version_overwrite = version

            if self.latest and name == "otxecm":
                version = version.rsplit(".", 1)[0]

            if self.latest and name == "otxecm-ctrl":
                version = version.rsplit(".", 1)[0] + "-latest"

            all_images.append(
                ContainerImage(
                    repository=self.repository,
                    path=self.path,
                    name=name,
                    version=version,
                    version_overwrite=version_overwrite,
                    docker_client=self.docker_client,
                )
            )

        if self.langpacks:
            langpacks_containers = [
                "otxecm-init-manifest",
                "otxecm-init-lang-ar",
                "otxecm-init-lang-ca-es",
                "otxecm-init-lang-cs-cz",
                "otxecm-init-lang-da-dk",
                "otxecm-init-lang-de",
                "otxecm-init-lang-es",
                "otxecm-init-lang-fi-fi",
                "otxecm-init-lang-fr",
                "otxecm-init-lang-he",
                "otxecm-init-lang-it",
                "otxecm-init-lang-iw",
                "otxecm-init-lang-ja",
                "otxecm-init-lang-kk-kz",
                "otxecm-init-lang-ko-kr",
                "otxecm-init-lang-nb-no",
                "otxecm-init-lang-nl",
                "otxecm-init-lang-pl-pl",
                "otxecm-init-lang-pt",
                "otxecm-init-lang-ru-ru",
                "otxecm-init-lang-sv",
                "otxecm-init-lang-tr-tr",
                "otxecm-init-lang-uk-ua",
                "otxecm-init-lang-zh-cn",
                "otxecm-init-lang-zh-tw",
            ]
            version = self.version.rsplit(".", 1)[0] + ".0"

            for image in langpacks_containers:
                all_images.append(
                    ContainerImage(
                        repository=self.repository,
                        path=self.path,
                        name=image,
                        version=version,
                        docker_client=self.docker_client,
                    )
                )

        return all_images



from .ContainerImage import ContainerImage
from .ContainerImageList import ContainerImageList
from .xECMImages import xECMImages
from .xECMMonImages import xECMMonImages
from .AppworksImages import AppworksImages
from .CSAIImages import CSAIImages
from .AppworksGatewayImages import AppworksGatewayImages



import docker.errors


class DockerException(docker.errors.DockerException):
    """
    Raised if a remote docker repository returns an error. Usually due to a failed push or pull.
    """

    def __init__(self, error_dict):
        self.response = error_dict["error"]
        self.explanation = error_dict["errorDetail"]["message"]



from operator import length_hint
from . import ContainerImage

from rich.progress import track, Progress, BarColumn, TextColumn


class ContainerImageList:
    def __init__(self, images, docker_client_args={}):
        self.images = images
        self.docker_client_args = docker_client_args

    def list(self):
        for image in self.images:
            image.print()

    def pull(self):
        with Progress() as progress:
            task = progress.add_task("Pulling ... ", total=len(self.images))
            for image in self.images:
                progress.console.print(f"Pulling {image.fullPath()}")
                image.pull()
                progress.advance(task)

    def push(self, repository, path, **kwargs):
        with Progress() as progress:
            task = progress.add_task(
                f"Publishing ...",
                total=len(self.images) * 2,
            )
            for image in self.images:
                progress.console.print(f"Pulling {image.fullPath()}", style="")
                image.pull()
                progress.advance(task)

                progress.console.print(f"Pushing {image.targetPath(repository, path)}")
                image.push(repository, path, **kwargs)
                progress.advance(task)

    def append(self, image: ContainerImage):
        self.images.append(image)

    def include(self, filter: str):
        filter_list = filter.split(",")  # Split the comma-separated filters into a list

        # Use list comprehension to filter self.images based on any of the filters
        self.images = [
            image for image in self.images if any(f in image.name for f in filter_list)
        ]

    def exclude(self, filter: str):
        filter_list = filter.split(",")  # Split the comma-separated filters into a list

        # Use list comprehension to filter self.images based on any of the filters
        self.images = [
            image
            for image in self.images
            if not any(f in image.name for f in filter_list)
        ]



import json
import logging
import os
from rich import console
from docker import DockerClient
from ..Exceptions.DockerException import DockerException


logger = logging.getLogger("otimages." + os.path.basename(__file__))

console = console.Console()


class ContainerImage:
    def __init__(
        self,
        docker_client: DockerClient,
        repository: str = "artifactory.otxlab.net",
        path: str = "",
        name: str = "",
        version: str = "",
        name_overwrite: str = "",
        version_overwrite: str = "",
    ):
        """CotainerImage class that can be used to pull and push Images to a different repo.

        Args:
            repository (str, optional): _description_. Defaults to "artifactory.otxlab.net".
            path (str, optional): _description_. Defaults to "".
            name (str, optional): _description_. Defaults to "".
            version (str, optional): _description_. Defaults to "".
            name_overwrite (str, optional): _description_. Defaults to "".
            version_overwrite (str, optional): _description_. Defaults to "".
        """
        self.repository = repository
        self.path = path[:-1] if path.endswith('/') else path
        self.name = name
        self.version = version.strip()
        self.pulled = False
        self.docker_client = docker_client

        if name_overwrite == "":
            self.name_overwrite = name
        else:
            self.name_overwrite = name_overwrite

        if version_overwrite == "":
            self.version_overwrite = version
        else:
            self.version_overwrite = version_overwrite

        self.image = None

        logger.debug("ContainerImage defined: %s", self.fullPath())

    def __str__(self) -> str:
        return self.fullPath()

    def fullPath(self) -> str:
        return f"{self.repository}{self.path}/{self.name}:{self.version}"

    def targetPath(self, repository: str, path: str = "") -> str:
        if path == "":
            target = f"{repository}/{self.name_overwrite}:{self.version_overwrite}"
        else:
            target = (
                f"{repository}/{path}/{self.name_overwrite}:{self.version_overwrite}"
            )
        target = target.strip()
        return target

    def pull(self) -> bool:
        logger.info("Pulling image: %s", self.fullPath())

        try:
            self.image = self.docker_client.images.pull(repository=self.fullPath())

        except NameError:
            logger.error("Run DockerInit() before working with images")
            exit()
        except Exception as err:
            self.pulled = False
            logger.error("Pulling of image {self.fullPath()} failed. %s", err)
        else:
            self.pulled = True

        return self.pulled

    def push(self, repository: str, path: str = "", **kwargs) -> bool:
        if not self.pulled:
            logger.debug("Image not yet pulled. Starting pull")
            if not self.pull():
                return False

        target = self.targetPath(repository, path)

        logger.info("Pushing image")
        logger.info("from: %s", self.fullPath())
        logger.info("to:   %s", target)

        try:
            self.image.tag(target)

            push_output = self.docker_client.images.push(target, **kwargs)
            for line in push_output.splitlines():
                parsed_line = json.loads(line)
                if "error" in parsed_line:
                    raise DockerException(parsed_line)

        except Exception as err:
            logger.error("Pushing of image %s failed. %s", target, err)
            return False

        return True

    def print(self):
        console.print(f"{self.repository}{self.path}/", end="")
        console.print(self.name, end="", style="red")
        console.print(":", end="")
        console.print(self.version)



import logging
import os
import subprocess
import typer

logger = logging.getLogger("otimages." + os.path.basename(__file__))


def run_helm_command(command: str) -> list:
    logger.debug("%s", command)
    result = subprocess.run(
        command,
        text=True,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        shell=True,
    )
    images = result.stdout.splitlines()

    try:
        assert images != []
    except AssertionError:
        for error in result.stderr.splitlines():
            logger.error("%s", error)
        logger.critical("Could not read images from helm -> %s", command)

        raise typer.Exit(code=1)

    return images



import logging
from enum import Enum

import sys
import os
import typer
from docker import DockerClient
from rich import print
from typing_extensions import Annotated
from .utils import (
    AppworksImages,
    xECMImages,
    xECMMonImages,
    CSAIImages,
    AppworksGatewayImages,
)


logger = logging.getLogger("otimages")


class TyperLoggerHandler(logging.Handler):

    def emit(self, record: logging.LogRecord) -> None:
        fg = None
        bg = None
        if record.levelno == logging.DEBUG:
            fg = typer.colors.BLACK
        elif record.levelno == logging.INFO:
            fg = typer.colors.BRIGHT_BLUE
        elif record.levelno == logging.WARNING:
            fg = typer.colors.BRIGHT_MAGENTA
        elif record.levelno == logging.CRITICAL:
            fg = typer.colors.BRIGHT_RED
        elif record.levelno == logging.ERROR:
            fg = typer.colors.BLACK
            bg = typer.colors.RED
        typer.secho(self.format(record), bg=bg, fg=fg)


class HelmChart(str, Enum):
    otxecm = "otxecm"
    xecmmon = "xecm-mon"
    cmo = "cmo"
    otawp = "otawp"
    appworks = "appworks"
    csai = "csai"
    otag = "otag"


app = typer.Typer()

try:
    docker_client = DockerClient(base_url=os.getenv("DOCKER_HOST", None))
except Exception as exc:
    logger.error("Cannot connect to Docker Engine")
    sys.exit()

option_version = Annotated[
    str,
    typer.Option("--version", "-v", prompt=True, help="Version of the Helm Chart"),
]
option_chart = Annotated[
    HelmChart,
    typer.Option("--chart", "-c", case_sensitive=False, help="Which HelmChart to use"),
]
option_langpacks = Annotated[
    bool,
    typer.Option(
        "--langpacks", help="Include all available LanguagePack initContainers"
    ),
]
option_latest = Annotated[
    bool,
    typer.Option("--latest", help="Latest or by default Released images + HelmChart"),
]
option_confirm = Annotated[
    bool,
    typer.Option("--confirm", "-y", help="Assume all answers are yes"),
]
option_include = Annotated[
    str,
    typer.Option("--include", help="Include only images matching this filter"),
]
option_exclude = Annotated[
    str,
    typer.Option("--exclude", help="Exclude all images matchting this filter"),
]

option_debug = Annotated[
    bool,
    typer.Option(
        "--debug",
        "-d",
        help="Enable global DEBUG logging, must be put before command: 'otimages --debug list -c otxecm'",
    ),
]


@app.callback()
def debug_log(verbose: option_debug = False):
    lvl = logging.WARNING
    fmt = "%(message)s"
    if verbose:
        lvl = logging.DEBUG

    typer_handler = TyperLoggerHandler()

    logging.basicConfig(
        format="%(asctime)s %(levelname)s [%(name)s] %(message)s",
        datefmt="%d-%b-%Y %H:%M:%S",
        level=lvl,
        handlers=(typer_handler,),
    )


@app.command(name="list")
def list_images(
    version: option_version = "24.3.0",
    chart: option_chart = HelmChart.otxecm,
    langpacks: option_langpacks = False,
    latest: option_latest = False,
    include: option_include = None,
    exclude: option_exclude = None,
):
    match chart.value:
        case "otxecm":
            images = xECMImages(
                version=version,
                latest=latest,
                langpacks=langpacks,
                docker_client=docker_client,
            )

        case "xecm-mon" | "cmo":
            images = xECMMonImages(
                version=version, latest=latest, docker_client=docker_client
            )

        case "appworks" | "otawp":
            images = AppworksImages(version=version, docker_client=docker_client)

        case "csai":
            images = CSAIImages(
                version=version, latest=latest, docker_client=docker_client
            )

        case "otag":
            images = AppworksGatewayImages(
                version=version, latest=latest, docker_client=docker_client
            )

    if include:
        images.include(include)

    if exclude:
        images.exclude(exclude)

    images.list()

    return images


@app.command()
def pull(
    chart: option_chart = "otxecm",
    version: option_version = "24.2.0",
    langpacks: option_langpacks = False,
    latest: option_latest = False,
    confirm: option_confirm = False,
    include: option_include = None,
    exclude: option_exclude = None,
):
    # List images
    images = list_images(
        version=version,
        chart=chart,
        langpacks=langpacks,
        latest=latest,
        include=include,
        exclude=exclude,
    )

    # Ask for confirmation if not preconfrimed
    if not confirm:
        confirm = typer.confirm("Are you sure you want to pull?")
        if not confirm:
            raise typer.Abort()

    images.pull()


@app.command()
def push(
    chart: option_chart = HelmChart.otxecm,
    version: option_version = "24.2.0",
    repository: Annotated[
        str,
        typer.Option(
            "--repository",
            "-r",
            prompt=True,
            help="repository where the images will be pushed to",
        ),
    ] = "terrarium.azurecr.io",
    path: Annotated[
        str, typer.Option("-p", "--path", help="Root Path of the target repository")
    ] = "",
    langpacks: option_langpacks = False,
    latest: option_latest = False,
    confirm: option_confirm = False,
    include: option_include = None,
    exclude: option_exclude = None,
):
    # List images
    images = list_images(
        version=version,
        chart=chart,
        langpacks=langpacks,
        latest=latest,
        include=include,
        exclude=exclude,
    )

    if path == "" and latest and chart != "xecm-mon":
        path = "latest"

    print()
    print("Push Images to repository:")
    for image in images.images:
        print(image.targetPath(repository=repository, path=path))

    # Ask for confirmation if not preconfrimed
    if not confirm:
        confirm = typer.confirm(
            f"Are you sure you want to push to {repository}/{path}?"
        )
        if not confirm:
            raise typer.Abort()

    images.push(repository=repository, path=path)


if __name__ == "__main__":

    app()
